searchData={"items":[{"type":"behaviour","title":"ExMachina","doc":"Defines functions for generating data\n\nIn depth examples are in the [README](readme.html)","ref":"ExMachina.html"},{"type":"callback","title":"ExMachina.build/1","doc":"","ref":"ExMachina.html#c:build/1"},{"type":"callback","title":"ExMachina.build/2","doc":"","ref":"ExMachina.html#c:build/2"},{"type":"function","title":"ExMachina.build/3","doc":"Builds a single factory.\n\nThis will defer to the `[factory_name]_factory/0` callback defined in the\nfactory module in which it is `use`d.","ref":"ExMachina.html#build/3"},{"type":"function","title":"Example - ExMachina.build/3","doc":"def user_factory do\n      %{name: \"John Doe\", admin: false}\n    end\n\n    # Returns %{name: \"John Doe\", admin: false}\n    build(:user)\n\n    # Returns %{name: \"John Doe\", admin: true}\n    build(:user, admin: true)","ref":"ExMachina.html#build/3-example"},{"type":"function","title":"Full control of a factory's attributes - ExMachina.build/3","doc":"If you want full control over the factory attributes, you can define the\nfactory with `[factory_name]_factory/1`, taking in the attributes as the first\nargument.\n\nCaveats:\n\n- ExMachina will no longer merge the attributes for your factory. If you want\nto do that, you can merge the attributes with the `merge_attributes/2` helper.\n\n- ExMachina will no longer evaluate lazy attributes. If you want to do that,\nyou can evaluate the lazy attributes with the `evaluate_lazy_attributes/1`\nhelper.","ref":"ExMachina.html#build/3-full-control-of-a-factory-s-attributes"},{"type":"function","title":"Example - ExMachina.build/3","doc":"def article_factory(attrs) do\n      title = Map.get(attrs, :title, \"default title\")\n      slug = Article.title_to_slug(title)\n\n      article = %Article{title: title, slug: slug}\n\n      article\n      # merge attributes on your own\n      |> merge_attributes(attrs)\n      # evaluate any lazy attributes\n      |> evaluate_lazy_attributes()\n    end\n\n    # Returns %Article{title: \"default title\", slug: \"default-title\"}\n    build(:article)\n\n    # Returns %Article{title: \"hello world\", slug: \"hello-world\"}\n    build(:article, title: \"hello world\")","ref":"ExMachina.html#build/3-example"},{"type":"callback","title":"ExMachina.build_list/2","doc":"","ref":"ExMachina.html#c:build_list/2"},{"type":"callback","title":"ExMachina.build_list/3","doc":"","ref":"ExMachina.html#c:build_list/3"},{"type":"function","title":"ExMachina.build_list/4","doc":"Builds any number of factories.","ref":"ExMachina.html#build_list/4"},{"type":"function","title":"Example - ExMachina.build_list/4","doc":"# Returns a list of 3 users\n    build_list(3, :user)","ref":"ExMachina.html#build_list/4-example"},{"type":"callback","title":"ExMachina.build_pair/1","doc":"","ref":"ExMachina.html#c:build_pair/1"},{"type":"callback","title":"ExMachina.build_pair/2","doc":"","ref":"ExMachina.html#c:build_pair/2"},{"type":"function","title":"ExMachina.build_pair/3","doc":"Builds two factories.\n\nThis is just an alias for `build_list(2, factory_name, attrs)`.","ref":"ExMachina.html#build_pair/3"},{"type":"function","title":"Example - ExMachina.build_pair/3","doc":"# Returns a list of 2 users\n    build_pair(:user)","ref":"ExMachina.html#build_pair/3-example"},{"type":"function","title":"ExMachina.evaluate_lazy_attributes/1","doc":"Helper function to evaluate lazy attributes that are passed into a factory.","ref":"ExMachina.html#evaluate_lazy_attributes/1"},{"type":"function","title":"Example - ExMachina.evaluate_lazy_attributes/1","doc":"# custom factory\n    def article_factory(attrs) do\n      %{title: \"title\"}\n      |> merge_attributes(attrs)\n      |> evaluate_lazy_attributes()\n    end\n\n    def author_factory do\n      %{name: sequence(\"gandalf\")}\n    end\n\n    # => returns [\n    #  %{title: \"title\", author: %{name: \"gandalf0\"},\n    #  %{title: \"title\", author: %{name: \"gandalf0\"}\n    # ]\n    build_pair(:article, author: build(:author))\n\n    # => returns [\n    #  %{title: \"title\", author: %{name: \"gandalf0\"},\n    #  %{title: \"title\", author: %{name: \"gandalf1\"}\n    # ]\n    build_pair(:article, author: fn -> build(:author) end)","ref":"ExMachina.html#evaluate_lazy_attributes/1-example"},{"type":"function","title":"ExMachina.merge_attributes/2","doc":"Helper function to merge attributes into a factory that could be either a map\nor a struct.","ref":"ExMachina.html#merge_attributes/2"},{"type":"function","title":"Example - ExMachina.merge_attributes/2","doc":"# custom factory\n    def article_factory(attrs) do\n      title = Map.get(attrs, :title, \"default title\")\n\n      article = %Article{\n        title: title\n      }\n\n      merge_attributes(article, attrs)\n    end\n\nNote that when trying to merge attributes into a struct, this function will\nraise if one of the attributes is not defined in the struct.","ref":"ExMachina.html#merge_attributes/2-example"},{"type":"function","title":"ExMachina.sequence/1","doc":"Shortcut for creating unique string values.\n\nThis is automatically imported into a model factory when you `use ExMachina`.\n\nThis is equivalent to `sequence(name, &\"#{name}#{&1}\")`. If you need to\ncustomize the returned string, see `sequence/2`.\n\nNote that sequences keep growing and are *not* reset by ExMachina. Most of the\ntime you won't need to reset the sequence, but when you do need to reset them,\nyou can use `ExMachina.Sequence.reset/0`.","ref":"ExMachina.html#sequence/1"},{"type":"function","title":"Examples - ExMachina.sequence/1","doc":"def user_factory do\n      %User{\n        # Will generate \"username0\" then \"username1\", etc.\n        username: sequence(\"username\")\n      }\n    end\n\n    def article_factory do\n      %Article{\n        # Will generate \"Article Title0\" then \"Article Title1\", etc.\n        title: sequence(\"Article Title\")\n      }\n    end","ref":"ExMachina.html#sequence/1-examples"},{"type":"function","title":"ExMachina.sequence/2","doc":"Create sequences for generating unique values.\n\nThis is automatically imported into a model factory when you `use ExMachina`.\n\nThe `name` can be any term, although it is typically an atom describing the\nsequence. Each time a sequence is called with the same `name`, its number is\nincremented by one.\n\nThe `formatter` function takes the sequence number, and returns a sequential\nrepresentation of that number – typically a formatted string.","ref":"ExMachina.html#sequence/2"},{"type":"function","title":"Examples - ExMachina.sequence/2","doc":"def user_factory do\n      %{\n        # Will generate \"me-0@foo.com\" then \"me-1@foo.com\", etc.\n        email: sequence(:email, &\"me-#{&1}@foo.com\"),\n        # Will generate \"admin\" then \"user\", \"other\", \"admin\" etc.\n        role: sequence(:role, [\"admin\", \"user\", \"other\"])\n      }\n    end","ref":"ExMachina.html#sequence/2-examples"},{"type":"function","title":"ExMachina.sequence/3","doc":"Similar to `sequence/2` but it allows for passing a `start_at` option\nto the sequence generation.","ref":"ExMachina.html#sequence/3"},{"type":"function","title":"Examples - ExMachina.sequence/3","doc":"def user_factory do\n      %{\n        # Will generate \"me-100@foo.com\" then \"me-101@foo.com\", etc.\n        email: sequence(:email, &\"me-#{&1}@foo.com\", start_at: 100),\n      }\n    end","ref":"ExMachina.html#sequence/3-examples"},{"type":"behaviour","title":"ExMachina.Ecto","doc":"Module for building and inserting factories with Ecto\n\nThis module works much like the regular `ExMachina` module, but adds a few\nnice things that make working with Ecto easier.\n\n* It uses `ExMachina.EctoStrategy`, which adds `insert/1`, `insert/2`,\n`insert/3` `insert_pair/2`, `insert_list/3`.\n* Adds a `params_for` function that is useful for working with changesets or\n  sending params to API endpoints.\n\nMore in-depth examples are in the [README](readme.html).","ref":"ExMachina.Ecto.html"},{"type":"callback","title":"ExMachina.Ecto.insert/1","doc":"","ref":"ExMachina.Ecto.html#c:insert/1"},{"type":"callback","title":"ExMachina.Ecto.insert/2","doc":"","ref":"ExMachina.Ecto.html#c:insert/2"},{"type":"callback","title":"ExMachina.Ecto.insert/3","doc":"Builds a factory and inserts it into the database.\n\nThe first two arguments are the same as `c:ExMachina.build/2`. The last\nargument is a set of options that will be passed to Ecto's\n[`Repo.insert!/2`](https://hexdocs.pm/ecto/Ecto.Repo.html#c:insert!/2).","ref":"ExMachina.Ecto.html#c:insert/3"},{"type":"callback","title":"Examples - ExMachina.Ecto.insert/3","doc":"# return all values from the database\n    insert(:user, [name: \"Jane\"], returning: true)\n    build(:user, name: \"Jane\") |> insert(returning: true)\n\n    # use a different prefix\n    insert(:user, [name: \"Jane\"], prefix: \"other_tenant\")\n    build(:user, name: \"Jane\") |> insert(prefix: \"other_tenant\")","ref":"ExMachina.Ecto.html#c:insert/3-examples"},{"type":"callback","title":"ExMachina.Ecto.insert_list/2","doc":"Builds many factories and inserts them into the database.\n\nThe arguments are the same as `c:ExMachina.build_list/3`.","ref":"ExMachina.Ecto.html#c:insert_list/2"},{"type":"callback","title":"ExMachina.Ecto.insert_list/3","doc":"","ref":"ExMachina.Ecto.html#c:insert_list/3"},{"type":"callback","title":"ExMachina.Ecto.insert_pair/1","doc":"Builds two factories and inserts them into the database.\n\nThe arguments are the same as `c:ExMachina.build_pair/2`.","ref":"ExMachina.Ecto.html#c:insert_pair/1"},{"type":"callback","title":"ExMachina.Ecto.insert_pair/2","doc":"","ref":"ExMachina.Ecto.html#c:insert_pair/2"},{"type":"callback","title":"ExMachina.Ecto.params_for/1","doc":"Builds a factory and returns only its fields.\n\nThis is only for use with Ecto models.\n\nWill return a map with the fields and virtual fields, but without the Ecto\nmetadata, the primary key, or any `belongs_to` associations. This will\nrecursively act on `has_one` associations and Ecto structs found in\n`has_many` associations.\n\nIf you want `belongs_to` associations to be inserted, use\n`c:params_with_assocs/2`.\n\nIf you want params with string keys use `c:string_params_for/2`.","ref":"ExMachina.Ecto.html#c:params_for/1"},{"type":"callback","title":"Example - ExMachina.Ecto.params_for/1","doc":"def user_factory do\n      %MyApp.User{name: \"John Doe\", admin: false}\n    end\n\n    # Returns %{name: \"John Doe\", admin: true}\n    params_for(:user, admin: true)\n\n    # Returns %{name: \"John Doe\", admin: false}\n    params_for(:user)","ref":"ExMachina.Ecto.html#c:params_for/1-example"},{"type":"callback","title":"ExMachina.Ecto.params_for/2","doc":"","ref":"ExMachina.Ecto.html#c:params_for/2"},{"type":"callback","title":"ExMachina.Ecto.params_with_assocs/1","doc":"Similar to `c:params_for/2` but inserts all `belongs_to` associations and\nsets the foreign keys.\n\nIf you want params with string keys use `c:string_params_with_assocs/2`.","ref":"ExMachina.Ecto.html#c:params_with_assocs/1"},{"type":"callback","title":"Example - ExMachina.Ecto.params_with_assocs/1","doc":"def article_factory do\n      %MyApp.Article{title: \"An Awesome Article\", author: build(:author)}\n    end\n\n    # Inserts an author and returns %{title: \"An Awesome Article\", author_id: 12}\n    params_with_assocs(:article)","ref":"ExMachina.Ecto.html#c:params_with_assocs/1-example"},{"type":"callback","title":"ExMachina.Ecto.params_with_assocs/2","doc":"","ref":"ExMachina.Ecto.html#c:params_with_assocs/2"},{"type":"callback","title":"ExMachina.Ecto.string_params_for/1","doc":"Similar to `c:params_for/2` but converts atom keys to strings in returned map.\n\nThe result of this function can be safely used in controller tests for Phoenix\nweb applications.","ref":"ExMachina.Ecto.html#c:string_params_for/1"},{"type":"callback","title":"Example - ExMachina.Ecto.string_params_for/1","doc":"def user_factory do\n      %MyApp.User{name: \"John Doe\", admin: false}\n    end\n\n    # Returns %{\"name\" => \"John Doe\", \"admin\" => true}\n    string_params_for(:user, admin: true)","ref":"ExMachina.Ecto.html#c:string_params_for/1-example"},{"type":"callback","title":"ExMachina.Ecto.string_params_for/2","doc":"","ref":"ExMachina.Ecto.html#c:string_params_for/2"},{"type":"callback","title":"ExMachina.Ecto.string_params_with_assocs/1","doc":"Similar to `c:params_with_assocs/2` but converts atom keys to strings in\nreturned map.\n\nThe result of this function can be safely used in controller tests for Phoenix\nweb applications.","ref":"ExMachina.Ecto.html#c:string_params_with_assocs/1"},{"type":"callback","title":"Example - ExMachina.Ecto.string_params_with_assocs/1","doc":"def article_factory do\n      %MyApp.Article{title: \"An Awesome Article\", author: build(:author)}\n    end\n\n    # Inserts an author and returns %{\"title\" => \"An Awesome Article\", \"author_id\" => 12}\n    string_params_with_assocs(:article)","ref":"ExMachina.Ecto.html#c:string_params_with_assocs/1-example"},{"type":"callback","title":"ExMachina.Ecto.string_params_with_assocs/2","doc":"","ref":"ExMachina.Ecto.html#c:string_params_with_assocs/2"},{"type":"module","title":"ExMachina.Sequence","doc":"Module for generating sequential values.\n\nUse `ExMachina.sequence/1` or `ExMachina.sequence/2` to generate\nsequential values instead of calling this module directly.","ref":"ExMachina.Sequence.html"},{"type":"function","title":"ExMachina.Sequence.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExMachina.Sequence.html#child_spec/1"},{"type":"function","title":"ExMachina.Sequence.reset/0","doc":"Reset all sequences so that the next sequence starts from 0","ref":"ExMachina.Sequence.html#reset/0"},{"type":"function","title":"Example - ExMachina.Sequence.reset/0","doc":"ExMachina.Sequence.next(\"joe\") # \"joe0\"\n    ExMachina.Sequence.next(\"joe\") # \"joe1\"\n\n    ExMachina.Sequence.reset()\n\n    ExMachina.Sequence.next(\"joe\") # resets so the return value is \"joe0\"\n\nYou can use list as well\n\n    ExMachina.Sequence.next(\"alphabet_sequence\", [\"A\", \"B\"]) # \"A\"\n    ExMachina.Sequence.next(\"alphabet_sequence\", [\"A\", \"B\"]) # \"B\"\n\n    ExMachina.Sequence.reset()\n\n    ExMachina.Sequence.next(\"alphabet_sequence\", [\"A\", \"B\"]) # resets so the return value is \"A\"\n\nIf you want to reset sequences at the beginning of every test, put it in a\n`setup` block in your test.\n\n    setup do\n      ExMachina.Sequence.reset()\n    end","ref":"ExMachina.Sequence.html#reset/0-example"},{"type":"function","title":"ExMachina.Sequence.reset/1","doc":"Reset specific sequences so long as they already exist. The sequences\nspecified will be reset to 0, while others will remain at their current index.\n\nYou can reset a single sequence,","ref":"ExMachina.Sequence.html#reset/1"},{"type":"function","title":"Example - ExMachina.Sequence.reset/1","doc":"ExMachina.Sequence.next(:alphabet, [\"A\", \"B\", \"C\"]) # \"A\"\n    ExMachina.Sequence.next(:alphabet, [\"A\", \"B\", \"C\"]) # \"B\"\n\n    ExMachina.Sequence.reset(:alphabet)\n\n    ExMachina.Sequence.next(:alphabet, [\"A\", \"B\", \"C\"]) # \"A\"\n\nAnd you can also reset multiple sequences at once,","ref":"ExMachina.Sequence.html#reset/1-example"},{"type":"function","title":"Example - ExMachina.Sequence.reset/1","doc":"ExMachina.Sequence.next(:numeric, [1, 2, 3]) # 1\n    ExMachina.Sequence.next(:numeric, [1, 2, 3]) # 2\n    ExMachina.Sequence.next(\"joe\") # \"joe0\"\n    ExMachina.Sequence.next(\"joe\") # \"joe1\"\n\n    ExMachina.Sequence.reset([\"joe\", :numeric])\n\n    ExMachina.Sequence.next(:numeric, [1, 2, 3]) # 1\n    ExMachina.Sequence.next(\"joe\") # \"joe0\"","ref":"ExMachina.Sequence.html#reset/1-example"},{"type":"module","title":"ExMachina.Strategy","doc":"Module for making new strategies for working with factories","ref":"ExMachina.Strategy.html"},{"type":"module","title":"Example - ExMachina.Strategy","doc":"defmodule MyApp.JsonEncodeStrategy do\n      # The function_name will be used to generate functions in your factory\n      # This example adds json_encode/1, json_encode/2, json_encode/3,\n      # json_encode_pair/2 and json_encode_list/3\n      use ExMachina.Strategy, function_name: :json_encode\n\n      # Define a function for handling the records.\n      # Takes the form of \"handle_#{function_name}\"\n      def handle_json_encode(record, %{encoder: encoder}) do\n        encoder.encode!(record)\n      end\n\n      # Optionally, define a function for handling records and taking in\n      # options at the function level\n      def handle_json_encode(record, %{encoder: encoder}, encoding_opts) do\n        encoder.encode!(record, encoding_opts)\n      end\n    end\n\n    defmodule MyApp.JsonFactory do\n      use ExMachina\n      use MyApp.JsonEncodeStrategy, encoder: Poison\n\n      def user_factory do\n        %User{name: \"John\"}\n      end\n    end\n\n    # Will build and then return a JSON encoded version of the user.\n    MyApp.JsonFactories.json_encode(:user)\n\nThe arguments sent to the handling function are\n\n  1. The built record\n  2. The options passed to the strategy\n  3. The options passed to the function as a third argument\n\nThe options sent as the second argument are always converted to a map. The\noptions are anything you passed when you `use` your strategy in your factory,\nmerged together with `%{factory_module: FactoryItWasCalledFrom}`.\n\nThis allows for customizing the strategy, and for calling other functions on\nthe factory if needed.\n\nSee `ExMachina.EctoStrategy` in the ExMachina repo, and the docs for\n`name_from_struct/1` for more examples.\n\nThe options sent as the third argument come directly from the options passed\nto the function being called. These could be function-level overrides of the\noptions passed when you `use` the strategy, or they could be other\ncustomizations needed at the level of the function.\n\nSee `c:ExMachina.Ecto.insert/3` for an example.","ref":"ExMachina.Strategy.html#module-example"},{"type":"function","title":"ExMachina.Strategy.name_from_struct/1","doc":"Returns the factory name from a struct. Useful for strategies with callbacks.\n\nThis function can be useful when you want to call other functions based on the\ntype of struct passed in. For example, if you wanted to call a function on the\nfactory module before JSON encoding.","ref":"ExMachina.Strategy.html#name_from_struct/1"},{"type":"function","title":"Examples - ExMachina.Strategy.name_from_struct/1","doc":"ExMachina.Strategy.name_from_struct(%User{}) # Returns :user\n    ExMachina.Strategy.name_from_struct(%MyUser{}) # Returns :my_user\n    ExMachina.Strategy.name_from_struct(%MyApp.MyTask{}) # Returns :my_task","ref":"ExMachina.Strategy.html#name_from_struct/1-examples"},{"type":"function","title":"Implementing callback functions with name_from_struct/1 - ExMachina.Strategy.name_from_struct/1","doc":"defmodule MyApp.JsonEncodeStrategy do\n      use ExMachina.Strategy, function_name: :json_encode\n\n      def handle_json_encode(record, %{factory_module: factory_module}) do\n        # If the record was a %User{} this would return :before_encode_user\n        callback_func_name = :\"before_encode_#{ExMachina.Strategy.name_from_struct(record)}\"\n\n        if callback_defined?(factory_module, callback_func_name) do\n          # First call the callback function\n          apply(factory_module, callback_func_name, [record])\n          # Then encode it\n          |> Poison.encode!\n        else\n          # Otherwise, encode it without calling any callback\n          Poison.encode!(record)\n        end\n      end\n\n      defp callback_defined?(module, func_name) do\n        Code.ensure_loaded?(module) && function_exported?(module, func_name, 1)\n      end\n    end","ref":"ExMachina.Strategy.html#name_from_struct/1-implementing-callback-functions-with-name_from_struct-1"},{"type":"exception","title":"ExMachina.UndefinedFactoryError","doc":"Error raised when trying to build or create a factory that is undefined.","ref":"ExMachina.UndefinedFactoryError.html"},{"type":"extras","title":"ExMachina","doc":"# ExMachina\n\n[![Continuous Integration](https://github.com/beam-community/ex_machina/actions/workflows/ci.yaml/badge.svg)](https://github.com/beam-community/ex_machina/actions/workflows/ci.yaml)\n[![Module Version](https://img.shields.io/hexpm/v/ex_machina.svg)](https://hex.pm/packages/ex_machina)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ex_machina/)\n[![Total Download](https://img.shields.io/hexpm/dt/ex_machina.svg)](https://hex.pm/packages/ex_machina)\n[![License](https://img.shields.io/hexpm/l/ex_machina.svg)](https://github.com/beam-community/ex_machina/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/beam-community/ex_machina.svg)](https://github.com/beam-community/ex_machina/commits/master)\n\nExMachina makes it easy to create test data and associations. It works great\nwith Ecto, but is configurable to work with any persistence library.\n\n> **This README follows the main branch, which may not be the currently published version**. Here are the\n[docs for the latest published version of ExMachina](https://hexdocs.pm/ex_machina/readme.html).","ref":"readme.html"},{"type":"extras","title":"Installation - ExMachina","doc":"In `mix.exs`, add the ExMachina dependency:\n\n<!-- {x-release-please-start-version} -->\n```elixir\ndef deps do\n  [\n    {:ex_machina, \"~> 2.8.0\", only: :test},\n  ]\nend\n```\n<!-- {x-release-please-end} -->\n\nAdd your factory module inside `test/support` so that it is only compiled in the\ntest environment.\n\nNext, be sure to start the application in your `test/test_helper.exs` before\nExUnit.start:\n\n```elixir\n{:ok, _} = Application.ensure_all_started(:ex_machina)\n```\n\n#### Install in just the test environment for non-Phoenix projects\n\nYou will follow the same instructions as above, but you will also need to add\n`test/support` to your compilation paths (elixirc_paths) if you have not done\nso already.\n\nIn `mix.exs`, add test/support to your elixirc_paths for just the test env.\n\n```elixir\ndef project do\n  [app: ...,\n   # Add this if it's not already in your project definition.\n   elixirc_paths: elixirc_paths(Mix.env)]\nend\n\n# This makes sure your factory and any other modules in test/support are compiled\n# when in the test environment.\ndefp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\ndefp elixirc_paths(_), do: [\"lib\"]\n```","ref":"readme.html#installation"},{"type":"extras","title":"Overview - ExMachina","doc":"[Check out the docs](https://hexdocs.pm/ex_machina/ExMachina.html) for more details.\n\nDefine factories:\n\n```elixir\ndefmodule MyApp.Factory do\n  # with Ecto\n  use ExMachina.Ecto, repo: MyApp.Repo\n\n  # without Ecto\n  use ExMachina\n\n  def user_factory do\n    %MyApp.User{\n      name: \"Jane Smith\",\n      email: sequence(:email, &\"email-#{&1}@example.com\"),\n      role: sequence(:role, [\"admin\", \"user\", \"other\"]),\n    }\n  end\n\n  def article_factory do\n    title = sequence(:title, &\"Use ExMachina! (Part #{&1})\")\n    # derived attribute\n    slug = MyApp.Article.title_to_slug(title)\n    %MyApp.Article{\n      title: title,\n      slug: slug,\n      # another way to build derived attributes\n      tags: fn article ->\n        if String.contains?(article.title, \"Silly\") do\n          [\"silly\"]\n        else\n          []\n        end\n      end,\n      # associations are inserted when you call `insert`\n      author: build(:user)\n    }\n  end\n\n  # derived factory\n  def featured_article_factory do\n    struct!(\n      article_factory(),\n      %{\n        featured: true,\n      }\n    )\n  end\n\n  def comment_factory do\n    %MyApp.Comment{\n      text: \"It's great!\",\n      article: build(:article),\n    }\n  end\nend\n```\n\nUsing factories ([check out the docs](https://hexdocs.pm/ex_machina/ExMachina.html) for more details):\n\n```elixir\n# `attrs` are automatically merged in for all build/insert functions.\n\n# `build*` returns an unsaved comment.\n# Associated records defined on the factory are built.\nattrs = %{body: \"A comment!\"} # attrs is optional. Also accepts a keyword list.\nbuild(:comment, attrs)\nbuild_pair(:comment, attrs)\nbuild_list(3, :comment, attrs)\n\n# `insert*` returns an inserted comment. Only works with ExMachina.Ecto\n# Associated records defined on the factory are inserted as well.\ninsert(:comment, attrs)\ninsert_pair(:comment, attrs)\ninsert_list(3, :comment, attrs)\n\n# `params_for` returns a plain map without any Ecto specific attributes.\n# This is only available when using `ExMachina.Ecto`.\nparams_for(:comment, attrs)\n\n# `params_with_assocs` is the same as `params_for` but inserts all belongs_to\n# associations and sets the foreign keys.\n# This is only available when using `ExMachina.Ecto`.\nparams_with_assocs(:comment, attrs)\n\n# Use `string_params_for` to generate maps with string keys. This can be useful\n# for Phoenix controller tests.\nstring_params_for(:comment, attrs)\nstring_params_with_assocs(:comment, attrs)\n```","ref":"readme.html#overview"},{"type":"extras","title":"Delayed evaluation of attributes - ExMachina","doc":"`build/2` is a function call. As such, it gets evaluated immediately. So this\ncode:\n\n```elixir\ninsert_pair(:account, user: build(:user))\n```\n\nIs equivalent to this:\n\n```elixir\nuser = build(:user)\ninsert_pair(:account, user: user) # same user for both accounts\n```\n\nSometimes that presents a problem. Consider the following factory:\n\n```elixir\ndef user_factory do\n  %{name: \"Gandalf\", email: sequence(:email, \"gandalf#{&1}@istari.com\")}\nend\n```\n\nIf you want to build a separate `user` per `account`, then calling\n`insert_pair(:account, user: build(:user))` will not give you the desired\nresult.\n\nIn those cases, you can delay the execution of the factory by passing it as an\nanonymous function:\n\n```elixir\ninsert_pair(:account, user: fn -> build(:user) end)\n```\n\nYou can also do that in a factory definition:\n\n```elixir\ndef account_factory do\n  %{user: fn -> build(:user) end}\nend\n```\n\nYou can even accept the parent record as an argument to the function:\n\n```elixir\ndef account_factory do\n  %{user: fn account -> build(:user, vip: account.premium) end}\nend\n```\n\nNote that the `account` passed to the anonymous function is only the struct\nafter it's built. It's not an inserted record. Thus, it does not have data that\nis only accessible after being inserted into the database (e.g. `id`).","ref":"readme.html#delayed-evaluation-of-attributes"},{"type":"extras","title":"Full control of factory - ExMachina","doc":"By default, ExMachina will merge the attributes you pass into build/insert into\nyour factory. But if you want full control of your attributes, you can define\nyour factory as accepting one argument, the attributes being passed into your\nfactory.\n\n```elixir\ndef custom_article_factory(attrs) do\n  title = Map.get(attrs, :title, \"default title\")\n\n  article = %Article{\n    author: \"John Doe\",\n    title: title\n  }\n\n  # merge attributes and evaluate lazy attributes at the end to emulate\n  # ExMachina's default behavior\n  article\n  |> merge_attributes(attrs)\n  |> evaluate_lazy_attributes()\nend\n```\n\n**NOTE** that in this case ExMachina will _not_ merge the attributes into your\nfactory, and it will not evaluate lazy attributes. You will have to do this on\nyour own if desired.","ref":"readme.html#full-control-of-factory"},{"type":"extras","title":"Non-map factories - ExMachina","doc":"Because you have full control of the factory when defining it with one argument,\nyou can build factories that are neither maps nor structs.\n\n```elixir\n# factory definition\ndef room_number_factory(attrs) do\n  %{floor: floor_number} = attrs\n  sequence(:room_number, &\"#{floor_number}0#{&1}\")\nend\n\n# example usage\nbuild(:room_number, floor: 5)\n# => \"500\"\n\nbuild(:room_number, floor: 5)\n# => \"501\"\n```\n\n**NOTE** that you cannot use non-map factories with Ecto. So you cannot\n`insert(:room_number)`.","ref":"readme.html#non-map-factories"},{"type":"extras","title":"Usage in a test - ExMachina","doc":"```elixir\n# Example of use in Phoenix with a factory that uses ExMachina.Ecto\ndefmodule MyApp.MyModuleTest do\n  use MyApp.ConnCase\n  # If using Phoenix, import this inside the using block in MyApp.ConnCase\n  import MyApp.Factory\n\n  test \"shows comments for an article\" do\n    conn = conn()\n    article = insert(:article)\n    comment = insert(:comment, article: article)\n\n    conn = get conn, article_path(conn, :show, article.id)\n\n    assert html_response(conn, 200) =~ article.title\n    assert html_response(conn, 200) =~ comment.body\n  end\nend\n```","ref":"readme.html#usage-in-a-test"},{"type":"extras","title":"Where to put your factories - ExMachina","doc":"If you are using ExMachina in all environments:\n\n> Start by creating one factory module (such as `MyApp.Factory`) in\n`lib/my_app/factory.ex` and putting all factory definitions in that module.\n\nIf you are using ExMachina in only the test environment:\n\n> Start by creating one factory module (such as `MyApp.Factory`) in\n`test/support/factory.ex` and putting all factory definitions in that module.\n\nLater on you can easily create different factories by creating a new module in\nthe same directory. This can be helpful if you need to create factories that are\nused for different repos, your factory module is getting too big, or if you have\ndifferent ways of saving the record for different types of factories.","ref":"readme.html#where-to-put-your-factories"},{"type":"extras","title":"Splitting factories into separate files - ExMachina","doc":"This example shows how to set up factories for the testing environment. For setting them in all environments, please see the _To install in all environments_ section\n\n> Start by creating main factory module in `test/support/factory.ex` and name it `MyApp.Factory`. The purpose of the main factory is to allow you to include only a single module in all tests.\n\n```elixir\n# test/support/factory.ex\ndefmodule MyApp.Factory do\n  use ExMachina.Ecto, repo: MyApp.Repo\n  use MyApp.ArticleFactory\nend\n```\n\nThe main factory includes `MyApp.ArticleFactory`, so let's create it next. It might be useful to create a separate directory for factories, like `test/factories`. Here is how to create a factory:\n\n```elixir\n# test/factories/article_factory.ex\ndefmodule MyApp.ArticleFactory do\n  defmacro __using__(_opts) do\n    quote do\n      def article_factory do\n        %MyApp.Article{\n          title: \"My awesome article!\",\n          body: \"Still working on it!\"\n        }\n      end\n    end\n  end\nend\n```\n\nThis way you can split your giant factory file into many small files. But what about name conflicts? Use pattern matching to avoid them!\n\n```elixir\n# test/factories/post_factory.ex\ndefmodule MyApp.PostFactory do\n  defmacro __using__(_opts) do\n    quote do\n      def post_factory do\n        %MyApp.Post{\n          body: \"Example body\"\n        }\n      end\n\n      def with_comments(%MyApp.Post{} = post) do\n        insert_pair(:comment, post: post)\n        post\n      end\n    end\n  end\nend\n\n# test/factories/video_factory.ex\ndefmodule MyApp.VideoFactory do\n  defmacro __using__(_opts) do\n    quote do\n      def video_factory do\n        %MyApp.Video{\n          url: \"example_url\"\n        }\n      end\n\n      def with_comments(%MyApp.Video{} = video) do\n        insert_pair(:comment, video: video)\n        video\n      end\n    end\n  end\nend\n```\n\nIf you place your factories outside of `test/support` make sure they will compile by adding that directory to the compilation paths in your `mix.exs` file. For example for the `test/factories` files above you would modify your file like so:\n\n```elixir\n# ./mix.exs\n...\n  defp elixirc_paths(:test), do: [\"lib\", \"test/factories\", \"test/support\"]\n...\n```","ref":"readme.html#splitting-factories-into-separate-files"},{"type":"extras","title":"Ecto - ExMachina","doc":"","ref":"readme.html#ecto"},{"type":"extras","title":"Ecto Associations - ExMachina","doc":"ExMachina will automatically save any associations when you call any of the\n`insert` functions. This includes `belongs_to` and anything that is\ninserted by Ecto when using `Repo.insert!`, such as `has_many`, `has_one`,\nand embeds. Since we automatically save these records for you, we advise that\nfactory definitions only use `build/2` when declaring associations, like so:\n\n```elixir\ndef article_factory do\n  %Article{\n    title: \"Use ExMachina!\",\n    # associations are inserted when you call `insert`\n    comments: [build(:comment)],\n    author: build(:user),\n  }\nend\n```\n\nUsing `insert/2` in factory definitions may lead to performance issues and bugs,\nas records will be saved unnecessarily.","ref":"readme.html#ecto-associations"},{"type":"extras","title":"Passing options to Repo.insert!/2 - ExMachina","doc":"`ExMachina.Ecto` uses\n[`Repo.insert!/2`](https://hexdocs.pm/ecto/Ecto.Repo.html#c:insert!/2) to\ninsert records into the database. Sometimes you may want to pass options to deal\nwith multi-tenancy or return some values generated by the database. In those\ncases, you can use `c:ExMachina.Ecto.insert/3`:\n\nFor example,\n\n```elixir\n# return values from the database\ninsert(:user, [name: \"Jane\"], returning: true)\n\n# use a different prefix\ninsert(:user, [name: \"Jane\"], prefix: \"other_tenant\")\n```","ref":"readme.html#passing-options-to-repo-insert-2"},{"type":"extras","title":"Flexible Factories with Pipes - ExMachina","doc":"```elixir\ndef make_admin(user) do\n  %{user | admin: true}\nend\n\ndef with_article(user) do\n  insert(:article, user: user)\n  user\nend\n\nbuild(:user) |> make_admin |> insert |> with_article\n```","ref":"readme.html#flexible-factories-with-pipes"},{"type":"extras","title":"Using with Phoenix - ExMachina","doc":"If you want to keep the factories somewhere other than `test/support`,\nchange this line in `mix.exs`:\n\n```elixir\n# Add the folder to the end of the list. In this case we're adding `test/factories`.\ndefp elixirc_paths(:test), do: [\"lib\", \"test/support\", \"test/factories\"]\n```","ref":"readme.html#using-with-phoenix"},{"type":"extras","title":"Custom Strategies - ExMachina","doc":"You can use ExMachina without Ecto, by using just the `build` functions, or you\ncan define one or more custom strategies to use in your factory. You can also\nuse custom strategies with Ecto. Here's an example of a strategy for json\nencoding your factories. See the docs on [ExMachina.Strategy] for more info.\n\n[ExMachina.Strategy]: https://hexdocs.pm/ex_machina/ExMachina.Strategy.html\n\n```elixir\ndefmodule MyApp.JsonEncodeStrategy do\n  use ExMachina.Strategy, function_name: :json_encode\n\n  def handle_json_encode(record, _opts) do\n    Poison.encode!(record)\n  end\nend\n\ndefmodule MyApp.Factory do\n  use ExMachina\n  # Using this will add json_encode/2, json_encode_pair/2 and json_encode_list/3\n  use MyApp.JsonEncodeStrategy\n\n  def user_factory do\n    %User{name: \"John\"}\n  end\nend\n\n# Will build and then return a JSON encoded version of the user.\nMyApp.Factory.json_encode(:user)\n```","ref":"readme.html#custom-strategies"},{"type":"extras","title":"Contributing - ExMachina","doc":"Before opening a pull request, please open an issue first.\n\n    git clone https://github.com/thoughtbot/ex_machina.git\n    cd ex_machina\n    mix deps.get\n    mix test\n\nOnce you've made your additions and `mix test` passes, go ahead and open a PR!","ref":"readme.html#contributing"},{"type":"extras","title":"License - ExMachina","doc":"ExMachina is Copyright © 2015 thoughtbot. It is free software, and may be\nredistributed under the terms specified in the [LICENSE](/LICENSE) file.","ref":"readme.html#license"},{"type":"extras","title":"About thoughtbot - ExMachina","doc":"![thoughtbot](https://thoughtbot.com/logo.png)\n\nExMachina is maintained and funded by thoughtbot, inc.\nThe names and logos for thoughtbot are trademarks of thoughtbot, inc.\n\nWe love open source software, Elixir, and Phoenix. See [our other Elixir\nprojects][elixir-phoenix], or [hire our Elixir Phoenix development team][hire]\nto design, develop, and grow your product.\n\n[elixir-phoenix]: https://thoughtbot.com/services/elixir-phoenix?utm_source=github\n[hire]: https://thoughtbot.com?utm_source=github","ref":"readme.html#about-thoughtbot"},{"type":"extras","title":"Inspiration - ExMachina","doc":"* [Fixtures for Ecto](https://blog.danielberkompas.com/elixir/2015/07/16/fixtures-for-ecto.html)\n* [Factory Bot](https://github.com/thoughtbot/factory_bot)","ref":"readme.html#inspiration"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThe noteworthy changes for each ExMachina version are included here. For a\ncomplete changelog, see the git history for each version via the version links.\n\n**To see the dates a version was published see the [hex package page].**\n\n[hex package page]: https://hex.pm/packages/ex_machina\n\n## [2.8.0](https://github.com/beam-community/ex_machina/compare/v2.7.0...v2.8.0) (2024-06-24)","ref":"changelog.html"},{"type":"extras","title":"Features - Changelog","doc":"* ExMachina.start/2: return a supervisor from Application callback ([#434](https://github.com/beam-community/ex_machina/issues/434)) ([c9ebb47](https://github.com/beam-community/ex_machina/commit/c9ebb47d7ffecdae3acf22f71c257de8bcdcffdc))","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Revert code changes breaking ecto record loading ([#447](https://github.com/beam-community/ex_machina/issues/447)) ([b796311](https://github.com/beam-community/ex_machina/commit/b796311761413086dac24dd3451ded8c0c349870))","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Miscellaneous - Changelog","doc":"* Add release please manifest file ([ae31578](https://github.com/beam-community/ex_machina/commit/ae31578057b682b6bf7d51d3bb29229e4cbb16c3))\n* Clear up log level warning ([821a61a](https://github.com/beam-community/ex_machina/commit/821a61a351676c2fee717451d8881059dc04730d))\n* Fix missing , in configuration file ([cf74a91](https://github.com/beam-community/ex_machina/commit/cf74a91a4f0b913b6b96a233692419a030be4cff))\n* README updates ([#444](https://github.com/beam-community/ex_machina/issues/444)) ([a4352dd](https://github.com/beam-community/ex_machina/commit/a4352dd59c3ce90d95fe85af65dda9ad332367d1))\n* Remove circleci, update tools-version ([#438](https://github.com/beam-community/ex_machina/issues/438)) ([b06f4b6](https://github.com/beam-community/ex_machina/commit/b06f4b6ebe323b8c0b4edcc961fbf4abb5a68bcd))\n* Remove extra character from test configuration file ([e8edf47](https://github.com/beam-community/ex_machina/commit/e8edf473c84ba414c5c07c267abac773cb64be6d))\n* Resolve config warning ([#440](https://github.com/beam-community/ex_machina/issues/440)) ([a327830](https://github.com/beam-community/ex_machina/commit/a32783031b21cd91bced561d4a30e9031f8a77e9))\n* Satisfy Credo consistency check ([5aa4f01](https://github.com/beam-community/ex_machina/commit/5aa4f01c8234a76cb3a41fa43e6da7cbd668ba03))\n* Support common-config ([#436](https://github.com/beam-community/ex_machina/issues/436)) ([2c2a309](https://github.com/beam-community/ex_machina/commit/2c2a309532f418083dac0df7bc74bf675056ad28))\n* Sync files with beam-community/common-config ([#437](https://github.com/beam-community/ex_machina/issues/437)) ([72e4038](https://github.com/beam-community/ex_machina/commit/72e40389a273a38be5f8dd96aef02976258212aa))\n* Sync files with beam-community/common-config ([#441](https://github.com/beam-community/ex_machina/issues/441)) ([c809bce](https://github.com/beam-community/ex_machina/commit/c809bce0db914604b431be8d47d922aae9fe8e3e))\n* Sync files with beam-community/common-config ([#448](https://github.com/beam-community/ex_machina/issues/448)) ([cca2acf](https://github.com/beam-community/ex_machina/commit/cca2acfecd66b1002eff2470a42e9302c76f5495))\n* Sync files with beam-community/common-config ([#450](https://github.com/beam-community/ex_machina/issues/450)) ([69612ae](https://github.com/beam-community/ex_machina/commit/69612ae19903a9410cc1fbaf9680d070c0b72370))\n* Update and run formatter ([#439](https://github.com/beam-community/ex_machina/issues/439)) ([8bb6057](https://github.com/beam-community/ex_machina/commit/8bb605725658a9dc36bd6e1f1579736f4b6514f4))\n* Update mix.exs and deps ([c6c76f0](https://github.com/beam-community/ex_machina/commit/c6c76f044d4fe8f57d82daed50800f8d43bd15b2))\n* Update test postgres configuration ([6aab2c8](https://github.com/beam-community/ex_machina/commit/6aab2c80cf17a66a0f087e6402b74e5477510884))\n\n## [2.7.0]\n\n[2.7.0]: https://github.com/thoughtbot/ex_machina/compare/v2.6.0...v2.7.0","ref":"changelog.html#miscellaneous"},{"type":"extras","title":"Added - Changelog","doc":"- Allow setting sequence starting point (#414)\n\n[#414]: https://github.com/thoughtbot/ex_machina/pull/414\n\n## [2.6.0]\n\n[2.6.0]: https://github.com/thoughtbot/ex_machina/compare/v2.5.0...v2.6.0","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Pass opts to Repo.insert! (add function-level opts to strategies) ([#411])","ref":"changelog.html#added"},{"type":"extras","title":"Fixes/Improvements - Changelog","doc":"- Import evaluate_lazy_attributes for ExMachina ([#410])","ref":"changelog.html#fixes-improvements"},{"type":"extras","title":"Docs - Changelog","doc":"- Use HTTPS for links in README ([#413])\n- Remove \"web\" dir from README.md ([#412])\n\n[#413]: https://github.com/thoughtbot/ex_machina/pull/413\n[#412]: https://github.com/thoughtbot/ex_machina/pull/412\n[#411]: https://github.com/thoughtbot/ex_machina/pull/411\n[#410]: https://github.com/thoughtbot/ex_machina/pull/410\n\n## [2.5.0]\n\n[2.5.0]: https://github.com/thoughtbot/ex_machina/compare/v2.4.0...v2.5.0","ref":"changelog.html#docs"},{"type":"extras","title":"Added - Changelog","doc":"- Allow delayed evaluation of attributes ([#408])","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix Elixir 1.11 compiler warnings ([#399])\n- Fix Elixir 1.11 warning by using extra_applications ([#400])","ref":"changelog.html#fixes"},{"type":"extras","title":"Docs - Changelog","doc":"- Update references to prior art ([#384])\n- Bump version number in Readme ([#376])\n\n[#376]: https://github.com/thoughtbot/ex_machina/pull/376\n[#384]: https://github.com/thoughtbot/ex_machina/pull/384\n[#399]: https://github.com/thoughtbot/ex_machina/pull/399\n[#400]: https://github.com/thoughtbot/ex_machina/pull/400\n[#408]: https://github.com/thoughtbot/ex_machina/pull/408\n\n## [2.4.0]","ref":"changelog.html#docs"},{"type":"extras","title":"Added - Changelog","doc":"- Allow ExMachina.Ecto to be used without :repo option ([#370])\n\n[2.4.0]: https://github.com/thoughtbot/ex_machina/compare/v2.3.0...v2.4.0\n[#370]: https://github.com/thoughtbot/ex_machina/pull/370\n\n## [2.3.0]","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Allows more control over factory definitions ([#333])\n- Adds ability to reset specific sequences ([#331])","ref":"changelog.html#added"},{"type":"extras","title":"Docs - Changelog","doc":"- Adds additional callbacks for functions with default params ([#319])","ref":"changelog.html#docs"},{"type":"extras","title":"Updated dependencies - Changelog","doc":"- Bump ex_doc from 0.19.1 to 0.19.3\n- Bump ecto_sql from 3.0.0 to 3.0.5\n- Bump ecto from 3.0.0 to 3.0.5\n\n[2.3.0]: https://github.com/thoughtbot/ex_machina/compare/v2.2.2...v2.3.0\n[#333]: https://github.com/thoughtbot/ex_machina/pull/333\n[#331]: https://github.com/thoughtbot/ex_machina/pull/331\n[#319]: https://github.com/thoughtbot/ex_machina/pull/319\n\n## [2.2.2]\n\n- Adds support for Ecto 3.0 ([#301])\n\n[2.2.2]: https://github.com/thoughtbot/ex_machina/compare/v2.2.1...v2.2.2\n[#301]: https://github.com/thoughtbot/ex_machina/pull/301\n\n## [2.2.1]","ref":"changelog.html#updated-dependencies"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixes sequence typespec ([#278])","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed `fields_for/2` function that would raise an error since 1.0.0 ([#287])","ref":"changelog.html#removed"},{"type":"extras","title":"Docs - Changelog","doc":"- Adds example for derived attribute ([#264])\n- Adds example for dependent factory ([#239])\n\n\n[2.2.1]: https://github.com/thoughtbot/ex_machina/compare/v2.2.0...v2.2.1\n[#239]: https://github.com/thoughtbot/ex_machina/pull/239\n[#264]: https://github.com/thoughtbot/ex_machina/pull/264\n[#278]: https://github.com/thoughtbot/ex_machina/pull/278\n[#287]: https://github.com/thoughtbot/ex_machina/pull/287\n\n## [2.2.0]","ref":"changelog.html#docs"},{"type":"extras","title":"Added - Changelog","doc":"- Adds support for using lists in sequences ([#227]).","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Elixir 1.6.x changed the behavior of `Regex.split/3` which caused factory\n  names to break. Added a fix in ([#275]).\n\n[2.2.0]: https://github.com/thoughtbot/ex_machina/compare/v2.1.0...v2.2.0\n[#227]: https://github.com/thoughtbot/ex_machina/pull/227\n[#275]: https://github.com/thoughtbot/ex_machina/pull/275\n\n## [2.1.0]","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Support bare maps in embeds https://github.com/thoughtbot/ex_machina/commit/efd4e7c6125843d20b8dd07d91ded6240ecaf5ef\n- Handle nested structures in `string_params_for/2` https://github.com/thoughtbot/ex_machina/pull/224","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Handle the number `0` in `*_list` functions https://github.com/thoughtbot/ex_machina/commit/012e957e7ab1e22eca18b62e8f3fcc2a98a7f286","ref":"changelog.html#fixed"},{"type":"extras","title":"Improved - Changelog","doc":"- Miscellaneous documentation improvements.\n\n[2.1.0]: https://github.com/thoughtbot/ex_machina/compare/v2.0.0...v2.1.0\n\n## [2.0.0]","ref":"changelog.html#improved"},{"type":"extras","title":"Added - Changelog","doc":"- Cast all values before insert ([#149])\n\n  For example, this means that if you have `field :equity, :decimal` in your\n  schema, you can set the value to `0` in your factory and it will automatically\n  cast the value to a Decimal.\n\n- Add `string_params_for`, which is useful for controller specs. ([#168])\n- Add `Sequence.reset/0` for resetting sequences between tests. ([#151])","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `params_*` functions now drop fields with `nil` values ([#148])\n- Don't delete `has_many`s from `params_*` functions ([#174])","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix an issue where values on embedded associations would not be cast ([#200])\n- Only drop autogenerated ids ([#147])\n- Fix an issue where setting an association to `nil` would break `insert` ([#193])\n- Fix an issue where unbuild has_many through associations were not removed in\n  `params_*` functions ([#192])\n\n[2.0.0]: https://github.com/thoughtbot/ex_machina/compare/v1.0.2...v2.0.0\n[#200]: https://github.com/thoughtbot/ex_machina/pull/200\n[#149]: https://github.com/thoughtbot/ex_machina/pull/149\n[#151]: https://github.com/thoughtbot/ex_machina/pull/151\n[#148]: https://github.com/thoughtbot/ex_machina/pull/148\n[#147]: https://github.com/thoughtbot/ex_machina/pull/147\n[#168]: https://github.com/thoughtbot/ex_machina/pull/168\n[#174]: https://github.com/thoughtbot/ex_machina/pull/174\n[#193]: https://github.com/thoughtbot/ex_machina/pull/193\n[#192]: https://github.com/thoughtbot/ex_machina/pull/192\n\n## [1.0.2]\n\nMinor documentation fixes\n\n[1.0.2]: https://github.com/thoughtbot/ex_machina/compare/v1.0.1...v1.0.2\n\n## [1.0.1]\n\nSmall change to the error generated when a factory definition is not found ([#142])\n\n[1.0.1]: https://github.com/thoughtbot/ex_machina/compare/v1.0.0...v1.0.1\n[#142]: https://github.com/thoughtbot/ex_machina/pull/142\n\n## [1.0.0]\n\nA lot has changed but we tried to make upgrading as simple as possible.\n\n**To upgrade:** In `mix.exs` change the version to `\"~> 1.0\"` and run `mix\ndeps.get`. Once you've updated, run `mix test` and ExMachina will raise errors\nthat show you what needs to change to work with 1.0.0.","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix compilation issues under OTP 19 ([#138])\n- Raise helpful error when trying to insert twice ([#128])","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Sequence.next/1` for quickly creating sequences. Example:\n  `sequence(\"username\")` will generate `\"username1\"`, then `\"username2\"` ([#84])\n- Raise if passing invalid keys to structs ([#99])\n- Add `params_with_assocs` ([#124])","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename `fields_for` to `params_for` ([#98])\n- If using ExMachina with Ecto, use `insert`, `insert_list` and `insert_pair`\n  instead of `create_*`\n- Instead of defining a custom `save_record`, you can now implement an\n  `ExMachina.Strategy`. See the documentation on hex.pm for more info ([#102])\n- Define factory as `user_factory` instead of `factory(:user)` ([#110]). See PR\n  and related issue for details on why this was changed.\n- `params_for` no longer returns the primary key ([#123])\n\n[1.0.0]: https://github.com/thoughtbot/ex_machina/compare/v0.6.1...v1.0.0\n[#138]: https://github.com/thoughtbot/ex_machina/pull/138\n[#128]: https://github.com/thoughtbot/ex_machina/pull/128\n[#84]: https://github.com/thoughtbot/ex_machina/pull/84\n[#99]: https://github.com/thoughtbot/ex_machina/pull/99\n[#124]: https://github.com/thoughtbot/ex_machina/pull/124\n[#98]: https://github.com/thoughtbot/ex_machina/pull/98\n[#102]: https://github.com/thoughtbot/ex_machina/pull/102\n[#110]: https://github.com/thoughtbot/ex_machina/pull/110\n[#123]: https://github.com/thoughtbot/ex_machina/pull/123\n\n## [0.6.1]\n\nRemoves warnings as reported by\nhttps://github.com/thoughtbot/ex_machina/issues/70. We recommend updating if you\nare using Ecto 1.1. There are no backward incompatible changes and no new\nfeatures.\n\n[0.6.1]: https://github.com/thoughtbot/ex_machina/compare/v0.6.0...v0.6.1\n\n## [0.6.0]\n\nYou can continue using ExMachina 0.5.0 if you are not ready for Ecto 1.1 yet.\nThere are no additional new features in this release.\n\n- Updated to use Ecto 1.1\n- Require Ecto 1.1\n\nThere are still some warnings that we need to fix for Ecto 1.1, but this release\nat least fixes the error that was caused when upgrading to Ecto 1.1.\n\n[0.6.0]: https://github.com/thoughtbot/ex_machina/compare/v0.5.0...v0.6.0\n\n## [0.5.0]","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Factories were simplified so that `attrs` is no longer required. See [70a0481] and [issue #56]\n- ExMachina.Ecto.assoc/3 was removed. You can now use build(:factory) instead. See discussion in [issue #56]","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Use association id as defined on the schema [7c67047]\n\n[issue #56]:https://github.com/thoughtbot/ex_machina/issues/56\n[70a0481]: https://github.com/thoughtbot/ex_machina/commit/70a04814aacc33b3c727e133f4bd6b03a8217731\n[7c67047]:https://github.com/thoughtbot/ex_machina/commit/7c6704706cffa7285a608049a1b1f10784790fdd\n[0.5.0]: https://github.com/thoughtbot/ex_machina/compare/v0.4.0...v0.5.0\n\n## [0.4.0]","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for `has_many` and `has_one` Ecto associations. See [1ff4198].","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Factories must now be defined with functions. See [59b7d23]\n\n[1ff4198]: https://github.com/thoughtbot/ex_machina/commit/1ff4198488caa8225563ec2d4262a6f42d7d29be\n[59b7d23]: https://github.com/thoughtbot/ex_machina/commit/59b7d23522d8ef4a3ae209f856b4d3c159de376e\n[0.4.0]: https://github.com/thoughtbot/ex_machina/compare/v0.3.0...v0.4.0\n\n## [0.3.0]","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `build_list` and `build_pair`. See [8f332ce].\n- Add a `create` method that takes a map. This allows you to chain functions\nlike: `build(:foo) |> make_admin |> create`. See [59cbef5].\n\n[8f332ce]: https://github.com/thoughtbot/ex_machina/commit/8f332ce0499f4e81f9dbb653fef3a6bc1e697cb6\n[59cbef5]: https://github.com/thoughtbot/ex_machina/commit/59cbef569d7740d2958653fe177790b0cb506ff6","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Factories must now be defined with a macro. See [03c41f6]\n- `belongs_to` associations are now built instead of created. See [b518285].\n\n[b518285]: https://github.com/thoughtbot/ex_machina/commit/b518285fa144459c36848bda5e72498914c19cdd\n[03c41f6]: https://github.com/thoughtbot/ex_machina/commit/03c41f64470423a168f91d40edcd91eb242c3c61\n[0.3.0]: https://github.com/thoughtbot/ex_machina/compare/v0.2.0...v0.3.0\n\n## [0.2.0]","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Ecto functionality was extracted to `ExMachina.Ecto`. See [270c19b].\n\n[270c19b]: https://github.com/thoughtbot/ex_machina/commit/270c19bbb805b7c62365612419410990f28c8baf\n[0.2.0]: https://github.com/thoughtbot/ex_machina/compare/v0.1.0...v0.2.0","ref":"changelog.html#changed"},{"type":"extras","title":"LICENSE","doc":"Copyright (c) 2015 thoughtbot, inc.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}